@@ -45,20 +45,21 @@ gboolean
 fop_cmp_eval(FilterExprNode *s, LogMessage **msgs, gint num_msg)
 {
   FilterCmp *self = (FilterCmp *) s;
-  SBGString *left_buf = sb_gstring_acquire();
-  SBGString *right_buf = sb_gstring_acquire();
+  ScratchBuffersMarker marker;
+  GString *left_buf = scratch_buffers_alloc_and_mark(&marker);
+  GString *right_buf = scratch_buffers_alloc();
   gboolean result = FALSE;
   gint cmp;
 
-  log_template_format_with_context(self->left, msgs, num_msg, NULL, LTZ_LOCAL, 0, NULL, sb_gstring_string(left_buf));
-  log_template_format_with_context(self->right, msgs, num_msg, NULL, LTZ_LOCAL, 0, NULL, sb_gstring_string(right_buf));
+  log_template_format_with_context(self->left, msgs, num_msg, NULL, LTZ_LOCAL, 0, NULL, left_buf);
+  log_template_format_with_context(self->right, msgs, num_msg, NULL, LTZ_LOCAL, 0, NULL, right_buf);
 
   if (self->cmp_op & FCMP_NUM)
     {
       gint l, r;
 
-      l = atoi(sb_gstring_string(left_buf)->str);
-      r = atoi(sb_gstring_string(right_buf)->str);
+      l = atoi(left_buf->str);
+      r = atoi(right_buf->str);
       if (l == r)
         cmp = 0;
       else if (l < r)
@@ -68,7 +69,7 @@ fop_cmp_eval(FilterExprNode *s, LogMessage **msgs, gint num_msg)
     }
   else
     {
-      cmp = strcmp(sb_gstring_string(left_buf)->str, sb_gstring_string(right_buf)->str);
+      cmp = strcmp(left_buf->str, right_buf->str);
     }
 
   if (cmp == 0)
@@ -84,8 +85,13 @@ fop_cmp_eval(FilterExprNode *s, LogMessage **msgs, gint num_msg)
       result = self->cmp_op & FCMP_GT || self->cmp_op == 0;
     }
 
-  sb_gstring_release(left_buf);
-  sb_gstring_release(right_buf);
+  msg_debug("cmp() evaluation started",
+            evt_tag_str("left", left_buf->str),
+            evt_tag_str("operator", self->super.type),
+            evt_tag_str("right", right_buf->str),
+            evt_tag_printf("msg", "%p", msgs[num_msg - 1]));
+
+  scratch_buffers_reclaim_marked(marker);
   return result ^ s->comp;
 }
 
@@ -108,7 +114,6 @@ fop_cmp_new(LogTemplate *left, LogTemplate *right, gint op)
   self->super.free_fn = fop_cmp_free;
   self->left = left;
   self->right = right;
-  self->super.type = "CMP";
 
   switch (op)
     {
@@ -116,37 +121,46 @@ fop_cmp_new(LogTemplate *left, LogTemplate *right, gint op)
       self->cmp_op = FCMP_NUM;
     case KW_LT:
       self->cmp_op |= FCMP_LT;
+      self->super.type = "<";
       break;
 
     case KW_NUM_LE:
       self->cmp_op = FCMP_NUM;
     case KW_LE:
       self->cmp_op |= FCMP_LT | FCMP_EQ;
+      self->super.type = "<=";
       break;
 
     case KW_NUM_EQ:
       self->cmp_op = FCMP_NUM;
     case KW_EQ:
       self->cmp_op |= FCMP_EQ;
+      self->super.type = "==";
       break;
 
     case KW_NUM_NE:
       self->cmp_op = FCMP_NUM;
     case KW_NE:
       self->cmp_op |= 0;
+      self->super.type = "!=";
       break;
 
     case KW_NUM_GE:
       self->cmp_op = FCMP_NUM;
     case KW_GE:
       self->cmp_op |= FCMP_GT | FCMP_EQ;
+      self->super.type = ">=";
       break;
 
     case KW_NUM_GT:
       self->cmp_op = FCMP_NUM;
     case KW_GT:
       self->cmp_op |= FCMP_GT;
+      self->super.type = ">";
       break;
+
+    default:
+      g_assert_not_reached();
     }
 
   if (self->cmp_op & FCMP_NUM && cfg_is_config_version_older(left->cfg, VERSION_VALUE_3_8))
