@@ -134,7 +134,7 @@ write_error:
     {
       msg_error("I/O error occurred while writing",
                 evt_tag_int("fd", self->super.transport->fd),
-                evt_tag_errno(EVT_TAG_OSERROR, errno));
+                evt_tag_error(EVT_TAG_OSERROR));
       return LPS_ERROR;
     }
 
@@ -154,7 +154,7 @@ write_error:
  * successfully sent this message, or if it should be resent by the caller.
  **/
 static LogProtoStatus
-log_proto_file_writer_post(LogProtoClient *s, guchar *msg, gsize msg_len, gboolean *consumed)
+log_proto_file_writer_post(LogProtoClient *s, LogMessage *logmsg, guchar *msg, gsize msg_len, gboolean *consumed)
 {
   LogProtoFileWriter *self = (LogProtoFileWriter *)s;
   LogProtoStatus result;
@@ -179,21 +179,20 @@ log_proto_file_writer_post(LogProtoClient *s, guchar *msg, gsize msg_len, gboole
   ++self->buf_count;
   self->sum_len += msg_len;
 
+  *consumed = TRUE;
+  log_proto_client_msg_ack(&self->super, 1);
+
   if (self->buf_count == self->buf_size)
     {
       /* we have reached the max buffer size -> we need to write the messages */
-      result = log_proto_file_writer_flush(s);
-      if (result != LPS_SUCCESS)
-        return result;
+      return log_proto_file_writer_flush(s);
     }
 
-  *consumed = TRUE;
-  log_proto_client_msg_ack(&self->super, 1);
   return LPS_SUCCESS;
 }
 
 static gboolean
-log_proto_file_writer_prepare(LogProtoClient *s, gint *fd, GIOCondition *cond)
+log_proto_file_writer_prepare(LogProtoClient *s, gint *fd, GIOCondition *cond, gint *timeout)
 {
   LogProtoFileWriter *self = (LogProtoFileWriter *) s;
 
@@ -220,7 +219,7 @@ log_proto_file_writer_new(LogTransport *transport, const LogProtoClientOptions *
 
   /* allocate the structure with the proper number of items at the end */
   LogProtoFileWriter *self = (LogProtoFileWriter *)g_malloc0(sizeof(LogProtoFileWriter) + sizeof(
-                               struct iovec)*flush_lines);
+      struct iovec)*flush_lines);
 
   log_proto_client_init(&self->super, transport, options);
   self->fd = transport->fd;
