@@ -39,7 +39,7 @@ struct _AFSocketDestDriver
   LogDestDriver super;
 
   gboolean
-    connections_kept_alive_accross_reloads:1;
+  connections_kept_alive_across_reloads:1;
   gint fd;
   LogWriter *writer;
   LogWriterOptions writer_options;
@@ -80,10 +80,15 @@ afsocket_dd_get_dest_name(const AFSocketDestDriver *s)
 LogWriter *afsocket_dd_construct_writer_method(AFSocketDestDriver *self);
 gboolean afsocket_dd_setup_addresses_method(AFSocketDestDriver *self);
 void afsocket_dd_set_keep_alive(LogDriver *self, gint enable);
-void afsocket_dd_init_instance(AFSocketDestDriver *self, SocketOptions *socket_options, TransportMapper *transport_mapper, GlobalConfig *cfg);
+void afsocket_dd_init_instance(AFSocketDestDriver *self, SocketOptions *socket_options,
+                               TransportMapper *transport_mapper, GlobalConfig *cfg);
 LogTransport *afsocket_dd_construct_transport_method(AFSocketDestDriver *self, gint fd);
+void afsocket_dd_reconnect(AFSocketDestDriver *self);
+void afsocket_dd_stop_watches(AFSocketDestDriver *self);
 
 gboolean afsocket_dd_init(LogPipe *s);
+gboolean afsocket_dd_deinit(LogPipe *s);
 void afsocket_dd_free(LogPipe *s);
+void afsocket_dd_connected_with_fd(gpointer self, gint fd, GSockAddr *saddr);
 
 #endif
