@@ -63,7 +63,7 @@ extern int afstreams_debug;
     LL_CONTEXT_INNER_SRC = 16,
     LL_CONTEXT_CLIENT_PROTO = 17,
     LL_CONTEXT_SERVER_PROTO = 18,
-    LL_CONTEXT_SELECTOR = 19,
+    LL_CONTEXT_OPTIONS = 19,
     KW_SOURCE = 10000,
     KW_FILTER = 10001,
     KW_PARSER = 10002,
@@ -74,8 +74,10 @@ extern int afstreams_debug;
     KW_BLOCK = 10007,
     KW_JUNCTION = 10008,
     KW_CHANNEL = 10009,
-    KW_INTERNAL = 10010,
-    KW_FILE = 10011,
+    KW_IF = 10010,
+    KW_ELSE = 10011,
+    KW_ELIF = 10012,
+    KW_INTERNAL = 10020,
     KW_SYSLOG = 10060,
     KW_MARK_FREQ = 10071,
     KW_STATS_FREQ = 10072,
@@ -121,7 +123,6 @@ extern int afstreams_debug;
     KW_THREADED = 10171,
     KW_PASS_UNIX_CREDENTIALS = 10231,
     KW_PERSIST_NAME = 10302,
-    KW_JVM_OPTIONS = 10303,
     KW_READ_OLD_RECORDS = 10304,
     KW_FLAGS = 10190,
     KW_PAD_SIZE = 10200,
@@ -163,15 +164,16 @@ extern int afstreams_debug;
     KW_IFDEF = 10410,
     KW_ENDIF = 10411,
     LL_DOTDOT = 10420,
-    LL_IDENTIFIER = 10421,
-    LL_NUMBER = 10422,
-    LL_FLOAT = 10423,
-    LL_STRING = 10424,
-    LL_TOKEN = 10425,
-    LL_BLOCK = 10426,
-    LL_PRAGMA = 10427,
-    LL_EOL = 10428,
-    LL_ERROR = 10429,
+    LL_DOTDOTDOT = 10421,
+    LL_IDENTIFIER = 10422,
+    LL_NUMBER = 10423,
+    LL_FLOAT = 10424,
+    LL_STRING = 10425,
+    LL_TOKEN = 10426,
+    LL_BLOCK = 10427,
+    LL_PRAGMA = 10428,
+    LL_EOL = 10429,
+    LL_ERROR = 10430,
     KW_VALUE_PAIRS = 10500,
     KW_EXCLUDE = 10502,
     KW_PAIR = 10503,
@@ -183,8 +185,8 @@ extern int afstreams_debug;
     KW_REPLACE_PREFIX = 10509,
     KW_ON_ERROR = 10510,
     KW_RETRIES = 10511,
-    KW_DOOR = 10020,
-    KW_SUN_STREAMS = 10021
+    KW_DOOR = 10513,
+    KW_SUN_STREAMS = 10514
   };
 #endif
 /* Tokens.  */
@@ -206,7 +208,7 @@ extern int afstreams_debug;
 #define LL_CONTEXT_INNER_SRC 16
 #define LL_CONTEXT_CLIENT_PROTO 17
 #define LL_CONTEXT_SERVER_PROTO 18
-#define LL_CONTEXT_SELECTOR 19
+#define LL_CONTEXT_OPTIONS 19
 #define KW_SOURCE 10000
 #define KW_FILTER 10001
 #define KW_PARSER 10002
@@ -217,8 +219,10 @@ extern int afstreams_debug;
 #define KW_BLOCK 10007
 #define KW_JUNCTION 10008
 #define KW_CHANNEL 10009
-#define KW_INTERNAL 10010
-#define KW_FILE 10011
+#define KW_IF 10010
+#define KW_ELSE 10011
+#define KW_ELIF 10012
+#define KW_INTERNAL 10020
 #define KW_SYSLOG 10060
 #define KW_MARK_FREQ 10071
 #define KW_STATS_FREQ 10072
@@ -264,7 +268,6 @@ extern int afstreams_debug;
 #define KW_THREADED 10171
 #define KW_PASS_UNIX_CREDENTIALS 10231
 #define KW_PERSIST_NAME 10302
-#define KW_JVM_OPTIONS 10303
 #define KW_READ_OLD_RECORDS 10304
 #define KW_FLAGS 10190
 #define KW_PAD_SIZE 10200
@@ -306,15 +309,16 @@ extern int afstreams_debug;
 #define KW_IFDEF 10410
 #define KW_ENDIF 10411
 #define LL_DOTDOT 10420
-#define LL_IDENTIFIER 10421
-#define LL_NUMBER 10422
-#define LL_FLOAT 10423
-#define LL_STRING 10424
-#define LL_TOKEN 10425
-#define LL_BLOCK 10426
-#define LL_PRAGMA 10427
-#define LL_EOL 10428
-#define LL_ERROR 10429
+#define LL_DOTDOTDOT 10421
+#define LL_IDENTIFIER 10422
+#define LL_NUMBER 10423
+#define LL_FLOAT 10424
+#define LL_STRING 10425
+#define LL_TOKEN 10426
+#define LL_BLOCK 10427
+#define LL_PRAGMA 10428
+#define LL_EOL 10429
+#define LL_ERROR 10430
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -326,8 +330,8 @@ extern int afstreams_debug;
 #define KW_REPLACE_PREFIX 10509
 #define KW_ON_ERROR 10510
 #define KW_RETRIES 10511
-#define KW_DOOR 10020
-#define KW_SUN_STREAMS 10021
+#define KW_DOOR 10513
+#define KW_SUN_STREAMS 10514
 
 /* Value type.  */
 
