@@ -38,12 +38,16 @@
  * include them all here. If it causes compilation times to increase
  * drastically, we should probably make them into separate compilation
  * units. (Bazsi) */
+#include "urlencode.c"
 #include "numeric-funcs.c"
 #include "str-funcs.c"
 #include "cond-funcs.c"
 #include "ip-funcs.c"
 #include "misc-funcs.c"
+#include "list-funcs.c"
 #include "tf-template.c"
+#include "context-funcs.c"
+#include "fname-funcs.c"
 
 static Plugin basicfuncs_plugins[] =
 {
@@ -52,6 +56,11 @@ static Plugin basicfuncs_plugins[] =
   TEMPLATE_FUNCTION_PLUGIN(tf_if, "if"),
   TEMPLATE_FUNCTION_PLUGIN(tf_or, "or"),
 
+  /* context related funcs */
+  TEMPLATE_FUNCTION_PLUGIN(tf_context_lookup, "context-lookup"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_context_length, "context-length"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_context_values, "context-values"),
+
   /* str-funcs */
   TEMPLATE_FUNCTION_PLUGIN(tf_echo, "echo"),
   TEMPLATE_FUNCTION_PLUGIN(tf_length, "length"),
@@ -62,6 +71,20 @@ static Plugin basicfuncs_plugins[] =
   TEMPLATE_FUNCTION_PLUGIN(tf_uppercase, "uppercase"),
   TEMPLATE_FUNCTION_PLUGIN(tf_replace_delimiter, "replace-delimiter"),
   TEMPLATE_FUNCTION_PLUGIN(tf_string_padding, "padding"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_binary, "binary"),
+
+  /* fname-funcs */
+  TEMPLATE_FUNCTION_PLUGIN(tf_dirname, "dirname"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_basename, "basename"),
+
+  /* list-funcs */
+  TEMPLATE_FUNCTION_PLUGIN(tf_list_concat, "list-concat"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_list_head, "list-head"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_list_nth, "list-nth"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_list_tail, "list-tail"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_list_slice, "list-slice"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_list_count, "list-count"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_list_append, "list-append"),
 
   /* numeric-funcs */
   TEMPLATE_FUNCTION_PLUGIN(tf_num_plus, "+"),
@@ -79,15 +102,15 @@ static Plugin basicfuncs_plugins[] =
   TEMPLATE_FUNCTION_PLUGIN(tf_indent_multi_line, "indent-multi-line"),
 
   /* misc funcs */
-  TEMPLATE_FUNCTION_PLUGIN(tf_context_length, "context-length"),
   TEMPLATE_FUNCTION_PLUGIN(tf_env, "env"),
-  TEMPLATE_FUNCTION_PLUGIN(tf_template, "template")
+  TEMPLATE_FUNCTION_PLUGIN(tf_template, "template"),
+  TEMPLATE_FUNCTION_PLUGIN(tf_urlencode, "urlencode")
 };
 
 gboolean
-basicfuncs_module_init(GlobalConfig *cfg, CfgArgs *args)
+basicfuncs_module_init(PluginContext *context, CfgArgs *args)
 {
-  plugin_register(cfg, basicfuncs_plugins, G_N_ELEMENTS(basicfuncs_plugins));
+  plugin_register(context, basicfuncs_plugins, G_N_ELEMENTS(basicfuncs_plugins));
   return TRUE;
 }
 
