@@ -26,7 +26,7 @@
 #include <ctype.h>
 
 static void
-tf_echo(LogMessage *msg, gint argc, GString *argv[], GString *result)
+_append_args_with_separator(gint argc, GString *argv[], GString *result, gchar separator)
 {
   gint i;
 
@@ -34,10 +34,17 @@ tf_echo(LogMessage *msg, gint argc, GString *argv[], GString *result)
     {
       g_string_append_len(result, argv[i]->str, argv[i]->len);
       if (i < argc - 1)
-        g_string_append_c(result, ' ');
+        g_string_append_c(result, separator);
     }
 }
 
+
+static void
+tf_echo(LogMessage *msg, gint argc, GString *argv[], GString *result)
+{
+  _append_args_with_separator(argc, argv, result, ' ');
+}
+
 TEMPLATE_FUNCTION_SIMPLE(tf_echo);
 
 static void
@@ -83,7 +90,8 @@ tf_substr(LogMessage *msg, gint argc, GString *argv[], GString *result)
   /* get offset position from second argument */
   if (!parse_number(argv[1]->str, &start))
     {
-      msg_error("$(substr) parsing failed, start could not be parsed", evt_tag_str("start", argv[1]->str));
+      msg_error("$(substr) parsing failed, start could not be parsed",
+                evt_tag_str("start", argv[1]->str));
       return;
     }
 
@@ -92,7 +100,8 @@ tf_substr(LogMessage *msg, gint argc, GString *argv[], GString *result)
     {
       if (!parse_number(argv[2]->str, &len))
         {
-          msg_error("$(substr) parsing failed, length could not be parsed", evt_tag_str("length", argv[2]->str));
+          msg_error("$(substr) parsing failed, length could not be parsed",
+                    evt_tag_str("length", argv[2]->str));
           return;
         }
     }
@@ -175,28 +184,24 @@ tf_strip(LogMessage *msg, gint argc, GString *argv[], GString *result)
 
   for (i = 0; i < argc; i++)
     {
-      gint spaces_end, spaces_start;
-
       if (argv[i]->len == 0)
         continue;
 
-      spaces_end = 0;
-      while (isspace(argv[i]->str[argv[i]->len - spaces_end - 1]))
+      gint spaces_end = 0;
+      while (isspace(argv[i]->str[argv[i]->len - spaces_end - 1]) && spaces_end < argv[i]->len)
         spaces_end++;
 
       if (argv[i]->len == spaces_end)
         continue;
 
-      spaces_start = 0;
+      gint spaces_start = 0;
       while (isspace(argv[i]->str[spaces_start]))
         spaces_start++;
 
-      if (argv[i]->len == spaces_start)
-        continue;
+      if (result->len > 0)
+        g_string_append_c(result, ' ');
 
       g_string_append_len(result, &argv[i]->str[spaces_start], argv[i]->len - spaces_end - spaces_start);
-      if (i < argc - 1)
-        g_string_append_c(result, ' ');
     }
 }
 
@@ -252,7 +257,6 @@ tf_sanitize_prepare(LogTemplateFunction *self, gpointer s, LogTemplate *parent,
 
   if (!tf_simple_func_prepare(self, state, parent, argc, argv, error))
     {
-      g_free(state);
       goto exit;
     }
   state->ctrl_chars = ctrl_chars;
@@ -307,26 +311,13 @@ TEMPLATE_FUNCTION(TFSanitizeState, tf_sanitize, tf_sanitize_prepare, tf_simple_f
                   tf_sanitize_free_state, NULL);
 
 
-static void
-append_args(gint argc, GString *argv[], GString *result)
-{
-  gint i;
-
-  for (i = 0; i < argc; i++)
-    {
-      g_string_append_len(result, argv[i]->str, argv[i]->len);
-      if (i < argc - 1)
-        g_string_append_c(result, ' ');
-    }
-}
-
 void
 tf_indent_multi_line(LogMessage *msg, gint argc, GString *argv[], GString *text)
 {
   gchar *p, *new_line;
 
   /* append the message text(s) to the template string */
-  append_args(argc, argv, text);
+  _append_args_with_separator(argc, argv, text, ' ');
 
   /* look up the \n-s and insert a \t after them */
   p = text->str;
@@ -403,44 +394,179 @@ tf_replace_delimiter(LogMessage *msg, gint argc, GString *argv[], GString *resul
 
 TEMPLATE_FUNCTION_SIMPLE(tf_replace_delimiter);
 
-static void
-tf_string_padding(LogMessage *msg, gint argc, GString *argv[], GString *result)
+typedef struct _TFStringPaddingState
 {
-  GString *text = argv[0];
-  GString *padding;
-  gint64 width, i;
+  TFSimpleFuncState super;
+  GString *padding_pattern;
+  gint64  width;
+} TFStringPaddingState;
 
-  if (argc <= 1)
+static gboolean
+_padding_prepare_parse_state(TFStringPaddingState *state, gint argc, gchar **argv, GError **error)
+{
+  if (argc < 3)
     {
-      msg_debug("Not enough arguments for padding template function!");
-      return;
+      g_set_error(error, LOG_TEMPLATE_ERROR, LOG_TEMPLATE_ERROR_COMPILE,
+                  "$(padding) Not enough arguments, usage $(padding <input> <width> [padding string])");
+      return FALSE;
     }
 
-  if (!parse_number_with_suffix(argv[1]->str, &width))
+  if (!parse_number(argv[2], &state->width))
     {
-      msg_debug("Padding template function requires a number as second argument!");
-      return;
-    }
 
-  if (argc <= 2)
-    padding = g_string_new(" ");
+      g_set_error(error, LOG_TEMPLATE_ERROR, LOG_TEMPLATE_ERROR_COMPILE,
+                  "Padding template function requires a number as second argument!");
+      return FALSE;
+    }
+  return TRUE;
+}
+
+static void
+_padding_prepare_fill_padding_pattern(TFStringPaddingState *state, gint argc, gchar **argv)
+{
+  state->padding_pattern = g_string_sized_new(state->width);
+  if (argc < 4)
+    {
+      g_string_printf(state->padding_pattern, "%*s", (int)(state->width), "");
+    }
   else
-    padding = argv[2];
-
-  if (text->len < width)
     {
-      for (i = 0; i < width - text->len; i++)
+      gint len = strlen(argv[3]);
+      if (len < 1)
         {
-          g_string_append_c(result, *(padding->str + (i % padding->len)));
+          g_string_printf(state->padding_pattern, "%*s", (int)(state->width), "");
+        }
+      else
+        {
+          gint repeat = state->width / len; // integer division!
+          for (gint i = 0; i < repeat; i++)
+            {
+              g_string_append_len(state->padding_pattern, argv[3], len);
+            }
+          g_string_append_len(state->padding_pattern, argv[3], state->width - (repeat * len));
         }
-    }
-
-  g_string_append_len(result, text->str, text->len);
-
-  if (argc <= 2)
-    {
-      g_string_free(padding, TRUE);
     }
 }
 
-TEMPLATE_FUNCTION_SIMPLE(tf_string_padding);
+static gboolean
+tf_string_padding_prepare(LogTemplateFunction *self, gpointer s, LogTemplate *parent,
+                          gint argc, gchar *argv[], GError **error)
+{
+  TFStringPaddingState *state = (TFStringPaddingState *) s;
+
+  if (!_padding_prepare_parse_state(state, argc, argv, error))
+    return FALSE;
+
+  _padding_prepare_fill_padding_pattern(state, argc, argv);
+
+  if (!tf_simple_func_prepare(self, state, parent, 2, argv, error))
+    {
+      g_set_error(error, LOG_TEMPLATE_ERROR, LOG_TEMPLATE_ERROR_COMPILE,
+                  "padding: prepare failed");
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
+static void
+tf_string_padding_call(LogTemplateFunction *self, gpointer s, const LogTemplateInvokeArgs *args, GString *result)
+{
+  TFStringPaddingState *state = (TFStringPaddingState *) s;
+  GString **argv = (GString **) args->bufs->pdata;
+
+  if (argv[0]->len > state->width)
+    {
+      g_string_append_len(result, argv[0]->str, argv[0]->len);
+    }
+  else
+    {
+      g_string_append_len(result, state->padding_pattern->str, state->width - argv[0]->len);
+      g_string_append_len(result, argv[0]->str, argv[0]->len);
+    }
+}
+
+static void
+tf_string_padding_free_state(gpointer s)
+{
+  TFStringPaddingState *state = (TFStringPaddingState *) s;
+  if (state->padding_pattern)
+    g_string_free(state->padding_pattern, TRUE);
+  tf_simple_func_free_state(&state->super);
+}
+
+TEMPLATE_FUNCTION(TFStringPaddingState, tf_string_padding, tf_string_padding_prepare,
+                  tf_simple_func_eval, tf_string_padding_call, tf_string_padding_free_state, NULL);
+
+typedef struct _TFBinaryState
+{
+  TFSimpleFuncState super;
+  GString *octets;
+} TFBinaryState;
+
+static void
+tf_binary_call(LogTemplateFunction *self, gpointer s, const LogTemplateInvokeArgs *args, GString *result)
+{
+  TFBinaryState *state = (TFBinaryState *) s;
+
+  g_string_append_len(result, state->octets->str, state->octets->len);
+}
+
+static gboolean
+tf_binary_prepare(LogTemplateFunction *self, gpointer s, LogTemplate *parent, gint argc, gchar *argv[], GError **error)
+{
+  TFBinaryState *state = (TFBinaryState *) s;
+  GString *octets = g_string_new("");
+
+  if (argc < 2)
+    {
+      g_set_error(error, LOG_TEMPLATE_ERROR, LOG_TEMPLATE_ERROR_COMPILE,
+                  "$(binary) Incorrect parameters, usage $(binary <number> <number> ...)");
+      goto error;
+    }
+
+  for (gint i = 1; i < argc; i++)
+    {
+      gint64 number;
+
+      gchar *token = argv[i];
+      if (!parse_number(token, &number))
+        {
+          g_set_error(error, LOG_TEMPLATE_ERROR, LOG_TEMPLATE_ERROR_COMPILE,
+                      "$(binary) template function requires list of dec/hex/oct numbers as arguments, unable to parse %s as a number",
+                      token);
+          goto error;
+        }
+      if (number > 0xFF)
+        {
+          g_set_error(error, LOG_TEMPLATE_ERROR, LOG_TEMPLATE_ERROR_COMPILE,
+                      "$(binary) template function only supports 8 bit values as characters, %" G_GUINT64_FORMAT " is above 255", number);
+          goto error;
+        }
+      g_string_append_c(octets, (gchar) number);
+    }
+
+  if (!tf_simple_func_prepare(self, state, parent, argc, argv, error))
+    {
+      goto error;
+    }
+
+  state->octets = octets;
+  return TRUE;
+error:
+  g_string_free(octets, TRUE);
+  return FALSE;
+}
+
+static void
+tf_binary_free_state(gpointer s)
+{
+  TFBinaryState *state = (TFBinaryState *) s;
+
+  if (state->octets)
+    g_string_free(state->octets, TRUE);
+  tf_simple_func_free_state(&state->super);
+}
+
+TEMPLATE_FUNCTION(TFBinaryState, tf_binary, tf_binary_prepare, tf_simple_func_eval, tf_binary_call,
+                  tf_binary_free_state, NULL);
