@@ -24,6 +24,7 @@
 #include "csvparser.h"
 #include "scanner/csv-scanner/csv-scanner.h"
 #include "parser/parser-expr.h"
+#include "scratch-buffers.h"
 
 #include <string.h>
 
@@ -31,8 +32,6 @@ typedef struct _CSVParser
 {
   LogParser super;
   CSVScannerOptions options;
-  CSVScanner scanner;
-  GString *formatted_key;
   gchar *prefix;
   gint prefix_len;
 } CSVParser;
@@ -98,16 +97,25 @@ csv_parser_set_prefix(LogParser *s, const gchar *prefix)
 }
 
 static const gchar *
-_get_formatted_key(CSVParser *self, const gchar *key)
+_format_key_for_prefix(GString *scratch, const gchar *key, const gint prefix_len)
 {
-  if (!self->prefix)
-    return key;
-  else if (self->formatted_key->len > 0)
-    g_string_truncate(self->formatted_key, self->prefix_len);
-  else
-    g_string_assign(self->formatted_key, self->prefix);
-  g_string_append(self->formatted_key, key);
-  return self->formatted_key->str;
+  g_string_truncate(scratch, prefix_len);
+  g_string_append(scratch, key);
+  return scratch->str;
+}
+
+static const gchar *
+_return_key(GString *scratch, const gchar *key, const gint prefix_len)
+{
+  return key;
+}
+
+typedef const gchar *(*key_formatter_t)(GString *scratch, const gchar *key, const gint prefix_len);
+
+static key_formatter_t
+dispatch_key_formatter(gchar *prefix)
+{
+  return prefix ? _format_key_for_prefix : _return_key;
 }
 
 static gboolean
@@ -117,17 +125,31 @@ csv_parser_process(LogParser *s, LogMessage **pmsg, const LogPathOptions *path_o
   CSVParser *self = (CSVParser *) s;
   LogMessage *msg = log_msg_make_writable(pmsg, path_options);
 
-  csv_scanner_input(&self->scanner, input);
-  while (csv_scanner_scan_next(&self->scanner))
+  msg_debug("csv-parser message processing started",
+            evt_tag_str ("input", input),
+            evt_tag_str ("prefix", self->prefix),
+            evt_tag_printf("msg", "%p", *pmsg));
+  CSVScanner scanner;
+  csv_scanner_init(&scanner, &self->options, input);
+
+  GString *key_scratch = scratch_buffers_alloc();
+  if (self->prefix)
+    g_string_assign(key_scratch, self->prefix);
+
+  key_formatter_t _key_formatter = dispatch_key_formatter(self->prefix);
+  while (csv_scanner_scan_next(&scanner))
     {
 
       log_msg_set_value_by_name(msg,
-                                _get_formatted_key(self, csv_scanner_get_current_name(&self->scanner)),
-                                csv_scanner_get_current_value(&self->scanner),
-                                csv_scanner_get_current_value_len(&self->scanner));
+                                _key_formatter(key_scratch, csv_scanner_get_current_name(&scanner), self->prefix_len),
+                                csv_scanner_get_current_value(&scanner),
+                                csv_scanner_get_current_value_len(&scanner));
     }
 
-  return csv_scanner_is_scan_finished(&self->scanner);
+  gboolean result = csv_scanner_is_scan_finished(&scanner);
+  csv_scanner_deinit(&scanner);
+
+  return result;
 }
 
 static LogPipe *
@@ -149,8 +171,6 @@ csv_parser_free(LogPipe *s)
   CSVParser *self = (CSVParser *) s;
 
   csv_scanner_options_clean(&self->options);
-  csv_scanner_state_clean(&self->scanner);
-  g_string_free(self->formatted_key, TRUE);
   g_free(self->prefix);
   log_parser_free_method(s);
 }
@@ -167,12 +187,10 @@ csv_parser_new(GlobalConfig *cfg)
   self->super.super.free_fn = csv_parser_free;
   self->super.super.clone = csv_parser_clone;
   self->super.process = csv_parser_process;
-  self->formatted_key = g_string_sized_new(32);
   csv_scanner_options_set_delimiters(&self->options, " ");
   csv_scanner_options_set_quote_pairs(&self->options, "\"\"''");
   csv_scanner_options_set_flags(&self->options, CSV_SCANNER_STRIP_WHITESPACE);
   csv_scanner_options_set_dialect(&self->options, CSV_SCANNER_ESCAPE_NONE);
-  csv_scanner_state_init(&self->scanner, &self->options);
   return &self->super;
 }
 
