@@ -155,8 +155,8 @@ _pop_state(PDBLoader *state)
 
 static gboolean
 _pop_state_for_closing_tag_with_alternatives(PDBLoader *state, const gchar *element_name,
-    const gchar *expected_element, const gchar *alternatives,
-    GError **error)
+                                             const gchar *expected_element, const gchar *alternatives,
+                                             GError **error)
 {
   if (strcmp(element_name, expected_element) != 0)
     {
@@ -607,7 +607,7 @@ _pdbl_rule_end(PDBLoader *state, const gchar *element_name, GError **error)
       /* valid, but we don't do anything */
     }
   else if (_pop_state_for_closing_tag_with_alternatives(state, element_name, "rule",
-           "</patterns>, </description>, </tags>, </urls>, </values>", error) == 0)
+                                                        "</patterns>, </description>, </tags>, </urls>, </values>", error) == 0)
     {
       if (state->current_rule)
         {
@@ -1143,8 +1143,8 @@ pdb_rule_set_load(PDBRuleSet *self, GlobalConfig *cfg, const gchar *config, GLis
     {
       msg_error("Error opening classifier configuration file",
                 evt_tag_str(EVT_TAG_FILENAME, config),
-                evt_tag_errno(EVT_TAG_OSERROR, errno));
-      goto error;
+                evt_tag_error(EVT_TAG_OSERROR));
+      return FALSE;
     }
 
   memset(&state, 0x0, sizeof(state));
