@@ -43,6 +43,7 @@
 #include "resolved-configurable-paths.h"
 #include "crypto.h"
 #include "compat/openssl_support.h"
+#include "scratch-buffers.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -54,7 +55,7 @@
 
 #define BOOL(x) ((x) ? "TRUE" : "FALSE")
 
-static gchar *full_colors[] =
+static const gchar *full_colors[] =
 {
   "\33[01;34m", /* blue */
   "\33[0;33m", /* brown */
@@ -62,7 +63,7 @@ static gchar *full_colors[] =
   "\33[01;31m"  /* red */
 };
 
-static gchar *empty_colors[] =
+static const gchar *empty_colors[] =
 {
   "", "", "", ""
 };
@@ -72,11 +73,11 @@ static gchar *empty_colors[] =
 #define COLOR_LITERAL 2
 #define COLOR_PARTIAL 3
 
-static gchar *no_color = "\33[01;0m";
-static gchar **colors = empty_colors;
+static const gchar *no_color = "\33[01;0m";
+static const gchar **colors = empty_colors;
 
 
-static gchar *patterndb_file = PATH_PATTERNDB_FILE;
+static const gchar *patterndb_file = PATH_PATTERNDB_FILE;
 static gboolean color_out = FALSE;
 
 typedef struct _PdbToolMergeState
@@ -419,7 +420,7 @@ pdbtool_match(int argc, char *argv[])
     {
       CfgLexer *lexer;
 
-      lexer = cfg_lexer_new_buffer(filter_string, strlen(filter_string));
+      lexer = cfg_lexer_new_buffer(configuration, filter_string, strlen(filter_string));
       if (!cfg_run_parser(configuration, lexer, &filter_expr_parser, (gpointer *) &filter, NULL))
         {
           fprintf(stderr, "Error parsing filter expression\n");
@@ -695,7 +696,7 @@ pdbtool_test_find_conflicts(PatternDB *patterndb, LogMessage *msg)
       gint matching_ids_len;
 
       matching_ids = r_find_all_applicable_nodes(program_rules->rules, (guint8 *) message, strlen(message),
-          (RNodeGetValueFunc) pdb_rule_get_name);
+                                                 (RNodeGetValueFunc) pdb_rule_get_name);
       matching_ids_len = g_strv_length(matching_ids);
 
       if (matching_ids_len > 1)
@@ -1023,7 +1024,7 @@ pdbtool_dictionary(int argc, char *argv[])
 static gboolean
 pdbtool_load_module(const gchar *option_name, const gchar *value, gpointer data, GError **error)
 {
-  return plugin_load_module(value, configuration, NULL);
+  return cfg_load_module(configuration, value);
 }
 
 static gchar *input_logfile = NULL;
@@ -1032,7 +1033,7 @@ static gdouble support_treshold = 4.0;
 static gboolean iterate_outliers = FALSE;
 static gboolean named_parsers = FALSE;
 static gint num_of_samples = 1;
-static gchar *delimiters = " :&~?![]=,;()'\"";
+static const gchar *delimiters = " :&~?![]=,;()'\"";
 
 static gint
 pdbtool_patternize(int argc, char *argv[])
@@ -1224,13 +1225,15 @@ main(int argc, char *argv[])
   msg_init(TRUE);
   resolved_configurable_paths_init(&resolvedConfigurablePaths);
   stats_init();
+  scratch_buffers_global_init();
+  scratch_buffers_allocator_init();
   log_msg_global_init();
   log_template_global_init();
   log_tags_global_init();
   pattern_db_global_init();
   crypto_init();
 
-  configuration = cfg_new(VERSION_VALUE);
+  configuration = cfg_new_snippet();
 
   if (!g_option_context_parse(ctx, &argc, &argv, &error))
     {
@@ -1241,13 +1244,15 @@ main(int argc, char *argv[])
     }
   g_option_context_free(ctx);
 
-  plugin_load_module("syslogformat", configuration, NULL);
-  plugin_load_module("basicfuncs", configuration, NULL);
+  cfg_load_module(configuration, "syslogformat");
+  cfg_load_module(configuration, "basicfuncs");
 
   if (color_out)
     colors = full_colors;
 
   ret = modes[mode].main(argc, argv);
+  scratch_buffers_allocator_deinit();
+  scratch_buffers_global_deinit();
   stats_destroy();
   log_tags_global_deinit();
   log_msg_global_deinit();
