@@ -118,9 +118,10 @@ python_fetch_debugger_command(void)
       msg_error("Error calling debugger fetch_command",
                 evt_tag_str("function", DEBUGGER_FETCH_COMMAND),
                 evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+      _py_finish_exception_handling();
       goto exit;
     }
-  if (!PyBytes_Check(ret))
+  if (!_py_is_string(ret))
     {
       msg_error("Return value from debugger fetch_command is not a string",
                 evt_tag_str("function", DEBUGGER_FETCH_COMMAND),
@@ -128,7 +129,7 @@ python_fetch_debugger_command(void)
       Py_DECREF(ret);
       goto exit;
     }
-  command = g_strdup(PyBytes_AsString(ret));
+  command = g_strdup(_py_get_string_as_string(ret));
   Py_DECREF(ret);
 exit:
   PyGILState_Release(gstate);
