@@ -56,14 +56,8 @@ _absolute_value(signed long diff)
     return diff;
 }
 
-/* This function determines the year that syslog-ng would find out
- * given the timestamp has no year information. Then returns the UTC
- * representation of "January 1st 00:00:00" of that year. This is to
- * be used for testcases that lack year information. ts_month is the 0
- * based month in the timestamp being parsed.
- */
 static time_t
-_get_bsd_year_utc(int ts_month)
+_get_epoch_with_bsd_year(int ts_month, int d, int h, int m, int s)
 {
   struct tm *tm;
   time_t t;
@@ -73,15 +67,16 @@ _get_bsd_year_utc(int ts_month)
 
   tm->tm_year = determine_year_for_month(ts_month, tm);
 
-  tm->tm_hour = 0;
-  tm->tm_min = 0;
-  tm->tm_sec = 0;
-  tm->tm_mday = 1;
-  tm->tm_mon = 0;
+  tm->tm_hour = h;
+  tm->tm_min = m;
+  tm->tm_sec = s;
+  tm->tm_mday = d;
+  tm->tm_mon = ts_month;
   tm->tm_isdst = -1;
   return mktime(tm);
 }
 
+
 static void
 _simulate_log_readers_effect_on_timezone_offset(LogMessage *message)
 {
@@ -152,7 +147,7 @@ void
 setup(void)
 {
   app_startup();
-  putenv("TZ=MET-1METDST");
+  setenv("TZ", "MET-1METDST", TRUE);
   tzset();
   init_and_load_syslogformat_module();
 }
@@ -236,7 +231,7 @@ Test(msgparse, test_failed_to_parse_too_long_sd_id)
       0, 0, 0,  // timestamp (sec/usec/zone)
       "", //host
       "syslog-ng", //app
-      "Error processing log message: <5>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [timeQuality isSynced=\"0\"][1234567890123456789012345678901234 i=\"long_33\"] An application event log entry...", // msg
+      "Error processing log message: <5>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [timeQuality isSynced=\"0\"][1234567890123456789012345678901>@<234 i=\"long_33\"] An application event log entry...", // msg
       "", // sd str,
       0, // processid
       0, // msgid,
@@ -259,7 +254,7 @@ Test(msgparse, test_bad_sd_data_unescaped)
       0, 0, 0,    // timestamp (sec/usec/zone)
       "",        // host
       "syslog-ng", //app
-      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [a i=\"\"ok\"] An application event log entry...", // msg
+      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [a i=\">@<\"ok\"] An application event log entry...", // msg
       "", //sd_str
       0,//processid
       0,//msgid
@@ -287,7 +282,7 @@ Test(msgparse, test_timestamp)
     {
       "<15>Jan  1 01:00:00 bzorp openvpn[2499]: PTHREAD support initialized", LP_EXPECT_HOSTNAME, NULL,
       15,             // pri
-      _get_bsd_year_utc(0) + 3600, 0, 3600,        // timestamp (sec/usec/zone)
+      _get_epoch_with_bsd_year(0, 1, 1, 0, 0), 0, 3600,        // timestamp (sec/usec/zone)
       "bzorp",        // host
       "openvpn",        // openvpn
       "PTHREAD support initialized", // msg
@@ -296,7 +291,8 @@ Test(msgparse, test_timestamp)
     {
       "<15>Jan 10 01:00:00 bzorp openvpn[2499]: PTHREAD support initialized", LP_EXPECT_HOSTNAME, NULL,
       15,             // pri
-      _get_bsd_year_utc(0) + 3600 + 9 * 24 * 3600, 0, 3600,        // timestamp (sec/usec/zone)
+      _get_epoch_with_bsd_year(0, 10, 1, 0, 0)
+      , 0, 3600,        // timestamp (sec/usec/zone)
       "bzorp",        // host
       "openvpn",        // openvpn
       "PTHREAD support initialized", // msg
@@ -305,7 +301,7 @@ Test(msgparse, test_timestamp)
     {
       "<13>Jan  1 14:40:51 alma korte: message", 0, NULL,
       13,
-      _get_bsd_year_utc(0) + 60 * 60 * 14 + 40 * 60 + 51, 0, 3600,
+      _get_epoch_with_bsd_year(0, 1, 14, 40, 51), 0, 3600,
       "",
       "alma",
       "korte: message",
@@ -562,7 +558,8 @@ Test(msgparse, test_timestamp_others)
     {
       "<190>NOV 22 00:00:33 192.168.33.8-1 CMDLOGGER[165319912]: cmd_logger_api.c(83) 13518 %% CLI:192.168.32.100:root:User  logged in", LP_EXPECT_HOSTNAME, NULL,
       190,
-      _get_bsd_year_utc(10) + 28166433, 0, 3600,
+      _get_epoch_with_bsd_year(10, 22, 0, 0, 33)
+      , 0, 3600,
       "192.168.33.8-1",
       "CMDLOGGER",
       "cmd_logger_api.c(83) 13518 %% CLI:192.168.32.100:root:User  logged in",
@@ -681,7 +678,7 @@ Test(msgparse, test_expected_sd_pairs_1)
       0, 0, 0,    // timestamp (sec/usec/zone)
       "",        // host
       "syslog-ng", //app
-      "Error processing log message: <7>1 2006-10-29T01:59:59.156Z mymachine.example.com evntslog - ID47 [ exampleSDID@0 iut=\"3\" eventSource=\"Application\" eventID=\"1011\"][examplePriority@0 class=\"high\"] \xEF\xBB\xBF" "An application event log entry...", // msg
+      "Error processing log message: <7>1 2006-10-29T01:59:59.156Z mymachine.example.com evntslog - ID47 >@<[ exampleSDID@0 iut=\"3\" eventSource=\"Application\" eventID=\"1011\"][examplePriority@0 class=\"high\"] \xEF\xBB\xBF" "An application event log entry...", // msg
       "",
       NULL,//processid
       NULL,//msgid
@@ -801,7 +798,7 @@ Test(msgparse, test_expected_sd_pairs_4)
       0, 0, 0,    // timestamp (sec/usec/zone)
       "",         // host
       "syslog-ng", //app
-      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa i=\"long_33\"] An application event log entry...", // msg
+      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa>@<aaa i=\"long_33\"] An application event log entry...", // msg
       "", //sd_str
       0,//processid
       0,//msgid
@@ -849,7 +846,7 @@ Test(msgparse, test_expected_sd_pairs_too_long)
       0, 0, 0,    // timestamp (sec/usec/zone)
       "",        // host
       "syslog-ng", //app
-      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [a aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa=\"long_33\"] An application event log entry...", // msg
+      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [a aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa>@<aaa=\"long_33\"] An application event log entry...", // msg
       "", //sd_str
       0,//processid
       0,//msgid
@@ -892,7 +889,7 @@ Test(msgparse, test_unescaped_too_long_message_parts)
       0, 0, 0,    // timestamp (sec/usec/zone)
       "", //host
       "syslog-ng", //app
-      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa evntslog - - [a i=\"ok\"] An application event log entry...",        // msg
+      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 >@<aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa evntslog - - [a i=\"ok\"] An application event log entry...",        // msg
       "", //sd_str
       0,//processid
       0,//msgid
@@ -948,7 +945,7 @@ Test(msgparse, test_unescaped_too_long_message_parts)
       0, 0, 0,    // timestamp (sec/usec/zone)
       "",        // host
       "syslog-ng", //app
-      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [a i=\"]ok\"] An application event log entry...", // msg
+      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [a i=\">@<]ok\"] An application event log entry...", // msg
       "", //sd_str
       0,//processid
       0,//msgid
@@ -963,7 +960,7 @@ Test(msgparse, test_unescaped_too_long_message_parts)
       0, 0, 0, // timestamp (sec/usec/zone)
       "",    // host
       "syslog-ng", //app
-      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [a i=\"\"ok\"] An application event log entry...", // msg
+      "Error processing log message: <132>1 2006-10-29T01:59:59.156+01:00 mymachine evntslog - - [a i=\">@<\"ok\"] An application event log entry...", // msg
       "", //sd_str
       0,//processid
       0,//msgid
@@ -1070,6 +1067,68 @@ Test(msgparse, test_expected_sd_pairs_without_sd_param)
   run_parameterized_test(params);
 }
 
+Test(msgparse, test_ip_in_host)
+{
+  struct msgparse_params params[] =
+  {
+    (struct msgparse_params)
+    {
+      .msg = "<0>Jan 10 01:00:00 1.2.3.4 prg0",
+      .parse_flags = LP_EXPECT_HOSTNAME,
+      .expected_stamp_sec =  _get_epoch_with_bsd_year(0, 10, 1, 0, 0),
+      .expected_stamp_ofs = 3600,
+      .expected_program = "prg0",
+      .expected_host = "1.2.3.4"
+    },
+    (struct msgparse_params)
+    {
+      .msg = "<0>Jan 10 01:00:00 0000:BABA:BA00:DAB:BABA:BABA:BABA:BAB0 prg0",
+      .parse_flags = LP_EXPECT_HOSTNAME,
+      .expected_stamp_sec =  _get_epoch_with_bsd_year(0, 10, 1, 0, 0),
+      .expected_stamp_ofs = 3600,
+      .expected_program = "prg0",
+      .expected_host = "0000:BABA:BA00:DAB:BABA:BABA:BABA:BAB0"
+    },
+    (struct msgparse_params)
+    {
+      .msg = "<0>Jan 10 01:00:00 0001:BABA:BA00:DAB::BAB0 prg0",
+      .parse_flags = LP_EXPECT_HOSTNAME,
+      .expected_stamp_sec =  _get_epoch_with_bsd_year(0, 10, 1, 0, 0),
+      .expected_stamp_ofs = 3600,
+      .expected_program = "prg0",
+      .expected_host = "0001:BABA:BA00:DAB::BAB0"
+    },
+    {
+      .msg = "<0>Jan 10 01:00:00 0002:: prg0: msgtxt",
+      .parse_flags = LP_EXPECT_HOSTNAME,
+      .expected_stamp_sec =  _get_epoch_with_bsd_year(0, 10, 1, 0, 0),
+      .expected_stamp_ofs = 3600,
+      .expected_program = "prg0",
+      .expected_host = "0002::",
+      .expected_msg = "msgtxt"
+    },
+    (struct msgparse_params)
+    {
+      .msg = "<0>Jan 10 01:00:00 prg0", // No ip no msg
+      .parse_flags = LP_EXPECT_HOSTNAME,
+      .expected_stamp_sec =  _get_epoch_with_bsd_year(0, 10, 1, 0, 0),
+      .expected_stamp_ofs = 3600,
+      .expected_program = "prg0",
+      .expected_host = ""
+    },
+    (struct msgparse_params)
+    {
+      .msg = "<0>Jan 10 01:00:00 prg0: msgtxt", // program name with message, no ip
+      .expected_stamp_sec =  _get_epoch_with_bsd_year(0, 10, 1, 0, 0),
+      .expected_stamp_ofs = 3600,
+      .expected_program = "prg0",
+      .expected_msg = "msgtxt"
+    },
+    {NULL}
+  };
+  run_parameterized_test(params);
+}
+
 Test(msgparse, test_simple_message)
 {
   struct msgparse_params params[] =
